""" Module for manipulating data from CORAL PRISM BIL files.

author: Joel Scott, NASA GSFC Ocean Ecology Lab / SAIC

Module includes:

class:    readL1B is designed to open and read the CORAL Project's PRISM BIL L1B files
          syntax: dataset = readL1B(dirname_untgz)

class:    readL2 is designed to open and read the CORAL Project's PRISM BIL L2 files
          syntax: dataset = readL1B(dirname_untgz)

class:    readL3 is designed to open and read the CORAL Project's PRISM BIL L3 files
          syntax: dataset = readL1B(dirname_untgz)

Notes:
* Compatibility: This module was developed for Python 3.X, using Python 3.6.2


Changelog:
    created 2017/09/24, jscott, originally developed for Python 3.6

"""

from collections import OrderedDict
from functools import reduce
import operator
import re
import glob
from statistics import mean
import numpy as np
from datetime import datetime, timedelta

#==========================================================================================================================================

def unix_time_sec(dt):
    """
    Return seconds since epoch (unix time).
    """
    epoch = datetime.utcfromtimestamp(0)

    return (dt - epoch).total_seconds()

#==========================================================================================================================================

def rd_prism_hdr(fname_hdr):
    """
    Read and parse PRISM HDR files
    """
    #define structure to hold hdr data
    dict_hdr = OrderedDict()
    
    #open and read hdr text data
    try:
        fid_hdr = open(fname_hdr,'r')

    except Exception as e:
        raise Exception('Unable to open HDR file for reading: {:}\n Error: {:}'.format(fname_hdr,e))
        return

    try:
        lines = fid_hdr.readlines()
        fid_hdr.close()

    except Exception as e:
        raise Exception('Unable to read data from HDR file: {:}\n Error: {:}'.format(fname_hdr,e))
        return

    lines = [re.sub("[\r\n]+",'',line).strip().lower() for line in lines]
    
    #parse and store start date/time
    ma = re.match("prm(\d{4})(\d{2})(\d{2})t(\d{2})(\d{2})(\d{2})\_([a-zA-Z]+)\_([a-zA-Z0-9]+)\_([a-zA-Z]+)(\_ort)?\.hdr",fname_hdr.rsplit('/')[-1])

    try:
        dict_hdr['start datetime'] = datetime(year = int(ma.group(1)),
                                              month = int(ma.group(2)),
                                              day = int(ma.group(3)),
                                              hour = int(ma.group(4)),
                                              minute = int(ma.group(5)),
                                              second = int(ma.group(6)))
        dict_hdr['start date'] = datetime(year = int(ma.group(1)),
                                          month = int(ma.group(2)),
                                          day = int(ma.group(3)))
        hdr_type = ma.group(7) + '_' + ma.group(9)
        dict_hdr['prism_version'] = ma.group(8)

    except Exception as e:
        raise Exception('Unable to parse date/time from filename: {:}\n Error: {:}'.format(fname_hdr.rsplit('/')[-1],e))
        return
    
    #parse and store hdr data
    flag_splitline = False
    line_sv = ''

    for line in lines:
        #flag lines that have content split between lines
        if '{' in line and not '}' in line:
            flag_splitline = True
            line_sv = line_sv + line
            continue
        
        #handle lines that have content split between lines
        if flag_splitline:
            line = line_sv
            flag_splitline = False
            line_sv = ''
        
        #create key-value pairs from hdr file
        try:
            [k,v] = line.split(' = ', 1)

        except:
            continue
        
        #handle key-value pairs, including where value is a list
        if '{' in v and '}' in v:
            ls_val = v[v.find('{')+1:v.rfind('}')].split(', ')
            dict_hdr[k] = [d.strip() for d in ls_val]

        else:
            dict_hdr[k] = v

    return[hdr_type, dict_hdr]

#==========================================================================================================================================

def rd_L1B_bil(fname_bil, dict_hdr):
    """
    Read and parse PRISM L1B BIL files
    """

    #define parameters to read binary data
    if 'img' in fname_bil.rsplit('/')[-1]:
        bil_type = '<f4' #little endian, single floating point, 32-bit

        if 'data ignore value' in dict_hdr:
            bil_miss = float(dict_hdr['data ignore value'])

    elif 'igm' in fname_bil.rsplit('/')[-1] or 'glt' in fname_bil.rsplit('/')[-1]:
        bil_type = '<i4' #little endian, signed long int, 32-bit
        if 'data ignore value' in dict_hdr:
            bil_miss = int(dict_hdr['data ignore value'])

    elif 'loc' in fname_bil.rsplit('/')[-1] or 'obs' in fname_bil.rsplit('/')[-1]:
        bil_type = '<f8' #little endian, double floating point, 64-bit
        if 'data ignore value' in dict_hdr:
            bil_miss = float(dict_hdr['data ignore value'])

    else:
        raise Exception('Unknown binary file precision for: {:}'.format(fname_bil))
        return
    
    bil_shape = (int(dict_hdr['lines']), int(dict_hdr['samples']), int(dict_hdr['bands']))
    bil_count = reduce(operator.mul, bil_shape, 1)
    
    #open and read binary data
    try:
        fid_bil = open(fname_bil,'r')

    except Exception as e:
        raise Exception('Unable to open BIL file for reading: {:}\n Error: {:}'.format(fname_bil,e))
        return

    try:
        bil_data = np.fromfile(fid_bil, dtype = bil_type, count = bil_count).reshape(bil_shape, order='C') #last index changes fastest, C-like array
        fid_bil.close()

    except Exception as e:
        raise Exception('Unable to read data from BIL file: {:}\n Error: {:}'.format(fname_bil,e))
        return

    dict_data = OrderedDict()

    #mask missing values
    if 'data ignore value' in dict_hdr:
        bil_data[np.where(bil_data == bil_miss)] = np.nan
    
    #parse and store binary data
    if 'band names' in dict_hdr:
        for k,var_name in zip(range(bil_shape[2]),dict_hdr['band names']):
            if 'utc time' in var_name:
                dict_data[var_name] = bil_data[:,:,k]

                #create arrays to store time in
                dict_data['scan_start_time'] = np.ndarray((bil_shape[0],), dtype="f8")
                dict_data['scan_end_time']   = np.ndarray((bil_shape[0],), dtype="f8")

                #convert time to year, day of year, and milliseconds
                for i, usec_lo, usec_hi in zip(range(bil_shape[0]), np.nanmin(bil_data[:,:,k], 1), np.nanmax(bil_data[:,:,k], 1)):
                    dt_scan_start = dict_hdr['start datetime'] + timedelta(microseconds = usec_lo)
                    dt_scan_end   = dict_hdr['start datetime'] + timedelta(microseconds = usec_hi)

                    dict_data['scan_start_time'][i] = unix_time_sec(dt_scan_start)
                    dict_data['scan_end_time'][i]   = unix_time_sec(dt_scan_end)

            else:
                dict_data[var_name] = bil_data[:,:,k]

    elif 'wavelength' in dict_hdr:
        #place wavelength in zero-th dim for nc output
        dict_data['lt'] = np.transpose(bil_data[:,:,:], (2, 0, 1))

    else:
        raise Exception('Unknown band key for: {:}'.format(fname_bil))
        return

    return dict_data

#==========================================================================================================================================

def rd_L2_bil(self, fname_bil, dict_hdr):
    """
    Read and parse PRISM L2 BIL files
    """
        
    #define parameters to read binary data
    bil_type = '<f4' #little endian, single floating point, 32-bit
    if 'data ignore value' in dict_hdr:
        bil_miss = float(dict_hdr['data ignore value'])
    
    bil_shape = (int(dict_hdr['lines']), int(dict_hdr['samples']), int(dict_hdr['bands']))
    bil_count = reduce(operator.mul, bil_shape, 1)
    
    #open and read binary data
    try:
        fid_bil = open(fname_bil,'r')

    except Exception as e:
        raise Exception('Unable to open BIL file for reading: {:}\n Error: {:}'.format(fname_bil,e))
        return

    try:
        bil_data = np.fromfile(fid_bil, dtype = bil_type, count = bil_count).reshape(bil_shape, order='C') #last index changes fastest, C-like array
        fid_bil.close()

    except Exception as e:
        raise Exception('Unable to read data from BIL file: {:}\n Error: {:}'.format(fname_bil,e))
        return

    #mask missing values
    if 'data ignore value' in dict_hdr:
        bil_data[np.where(bil_data == bil_miss)] = np.nan
    
    #parse and store binary data
    if '_corr_' in fname_bil.rsplit('/')[-1]:
        #place wavelength in zero-th dim for nc output
        #scale by Pi to get Rrs
        self.data['rrs'] = np.transpose(bil_data[:,:,:], (2, 0, 1)) / np.pi

    elif '_rb_' in fname_bil.rsplit('/')[-1]:
        #place wavelength in zero-th dim for nc output
        self.data['rb'] = np.transpose(bil_data[:,:,:], (2, 0, 1))

    elif '_dep_' in fname_bil.rsplit('/')[-1]:
        for k,var_name in zip(range(bil_shape[2]),dict_hdr['band names']):
            self.data[var_name] = bil_data[:,:,k]

    else:
        raise Exception('Unknown data/band/variable read from BIL file: {:}'.format(fname_bil))
        return

    return

#==========================================================================================================================================

def rd_L3_bil(self, fname_bil, dict_hdr):
    """
    Read and parse PRISM L3 BIL files
    """
        
    #define parameters to read binary data
    bil_type = '<f4' #little endian, single floating point, 32-bit
    if 'data ignore value' in dict_hdr:
        bil_miss = float(dict_hdr['data ignore value'])
    
    bil_shape = (int(dict_hdr['lines']), int(dict_hdr['samples']), int(dict_hdr['bands']))
    bil_count = reduce(operator.mul, bil_shape, 1)
    
    #open and read binary data
    try:
        fid_bil = open(fname_bil,'r')

    except Exception as e:
        raise Exception('Unable to open BIL file for reading: {:}\n Error: {:}'.format(fname_bil,e))
        return

    try:
        bil_data = np.fromfile(fid_bil, dtype = bil_type, count = bil_count).reshape(bil_shape, order='C') #last index changes fastest, C-like array
        fid_bil.close()

    except Exception as e:
        raise Exception('Unable to read data from BIL file: {:}\n Error: {:}'.format(fname_bil,e))
        return

    #mask missing values
    if 'data ignore value' in dict_hdr:
        bil_data[np.where(bil_data == bil_miss)] = np.nan
    
    #parse and store binary data
    for k,var_name in zip(range(bil_shape[2]),dict_hdr['band names']):
        self.data[var_name] = bil_data[:,:,k]

    return

#==========================================================================================================================================

class readL1B:
    """ 
        Read a CORAL L1B PRISM BIL file.

        Returned data structures:
        hdr       = dictionary of header contents, keyed by 
        data      = dictionary of data 2D numpy arrays, keyed by field name
    """

    def __init__(self, dirname_untgz):
        """
        Required arguments:
        dirname_untgz = directory name of the unzipped tgz bundle of L1B PRISM BIL and HDR files (string, no trailing slash)
        """

        self.hdr   = OrderedDict()
        self.data  = OrderedDict()

        ls_hdr = sorted(glob.glob(dirname_untgz + '/*.hdr'))
        ls_bil = [fname.rsplit('.')[0] for fname in ls_hdr]

        if not ls_hdr or not ls_bil:
            raise Exception('Missing PRISM header or BIL data in {:}'.format(dirname_untgz))
            return

        for fname_hdr,fname_bil in zip(ls_hdr,ls_bil):
            #skip files that are not Orthocorrected
            if 'loc.hdr' in fname_hdr or 'obs.hdr' in fname_hdr:
                continue

            [hdr_type, dict_hdr] = rd_prism_hdr(fname_hdr)

            #save all hdr content
            self.hdr[hdr_type] = OrderedDict()
            for key in dict_hdr:
                self.hdr[hdr_type][key] = dict_hdr[key]

            dict_data = rd_L1B_bil(fname_bil, dict_hdr)

            #save all data content
            self.data.update(dict_data)

        return

#==========================================================================================================================================

class readL2:
    """ 
        Read a CORAL L2 PRISM BIL file.

        Returned data structures:
        hdr       = dictionary of header contents, keyed by 
        data      = dictionary of data 2D numpy arrays, keyed by field name
    """

    def __init__(self, dirname_untgz):
        """
        Required arguments:
        dirname_untgz = directory name of the unzipped tgz bundle of L1B PRISM BIL and HDR files (string, no trailing slash)
        """

        self.hdr   = OrderedDict()
        self.data  = OrderedDict()

        ls_hdr = sorted(glob.glob(dirname_untgz + '/*.hdr'))
        ls_bil = [fname.rsplit('.')[0] for fname in ls_hdr]

        if not ls_hdr or not ls_bil:
            raise Exception('Missing PRISM header or BIL data in {:}'.format(dirname_untgz))
            return

        for fname_hdr,fname_bil in zip(ls_hdr,ls_bil):
            [hdr_type, dict_hdr] = rd_prism_hdr(fname_hdr)

            #save all hdr content
            self.hdr[hdr_type] = OrderedDict()
            for key in dict_hdr:
                self.hdr[hdr_type][key] = dict_hdr[key]

            rd_L2_bil(self, fname_bil, dict_hdr)

        return

#==========================================================================================================================================

class readL3:
    """ 
        Read a CORAL L3 PRISM BIL file.

        Returned data structures:
        hdr       = dictionary of header contents, keyed by 
        data      = dictionary of data 2D numpy arrays, keyed by field name
    """

    def __init__(self, dirname_untgz):
        """
        Required arguments:
        dirname_untgz = directory name of the unzipped tgz bundle of L1B PRISM BIL and HDR files (string, no trailing slash)
        """

        self.hdr   = OrderedDict()
        self.data  = OrderedDict()

        ls_hdr = sorted(glob.glob(dirname_untgz + '/*.hdr'))
        ls_bil = [fname.rsplit('.')[0] for fname in ls_hdr]

        if not ls_hdr or not ls_bil:
            raise Exception('Missing PRISM header or BIL data in {:}'.format(dirname_untgz))
            return

        for fname_hdr,fname_bil in zip(ls_hdr,ls_bil):
            [hdr_type, dict_hdr] = rd_prism_hdr(fname_hdr)

            #save all hdr content
            self.hdr[hdr_type] = OrderedDict()
            for key in dict_hdr:
                self.hdr[hdr_type][key] = dict_hdr[key]

            rd_L3_bil(self, fname_bil, dict_hdr)

        return
